@@ -35,28 +35,23 @@ public Q_SLOTS:
 
 private Q_SLOTS:
     void testWordWrap();
+    void testWrapParagraph();
     void testReplaceQStringList();
     void testMovingInterfaceSignals();
-
     void testSetTextPerformance();
     void testRemoveTextPerformance();
-
     void testForgivingApiUsage();
-
     void testRemoveMultipleLines();
     void testInsertNewline();
     void testInsertAfterEOF();
-
+    void testAutoBrackets();
     void testReplaceTabs();
-
     void testDigest();
     void testModelines();
-
     void testDefStyleNum();
-
     void testTypeCharsWithSurrogateAndNewLine();
-
     void testRemoveComposedCharacters();
+    void testAutoReload();
 };
 
 #endif // KATE_DOCUMENT_TEST_H
