@@ -42,40 +42,28 @@ class KatePrinter::PageLayout
 {
 public:
     PageLayout()
-        : pageWidth(0)
-        , pageHeight(0)
-        , headerWidth(0)
-        , maxWidth(0)
-        , maxHeight(0)
-        , xstart(0)
-        , innerMargin(0)
-        , selectionOnly(false)
-        , firstline(0)
-        , lastline(0)
-        , headerHeight(0)
-        , headerTagList()
-        , footerHeight(0)
+        : headerTagList()
         , footerTagList()
         , selectionRange()
     {}
 
-    uint pageWidth;
-    uint pageHeight;
-    uint headerWidth;
-    uint maxWidth;
-    uint maxHeight;
-    int xstart; // beginning point for painting lines
-    int innerMargin;
+    uint pageWidth = 0;
+    uint pageHeight = 0;
+    uint headerWidth = 0;
+    uint maxWidth = 0;
+    uint maxHeight = 0;
+    int xstart = 0; // beginning point for painting lines
+    int innerMargin = 0;
 
-    bool selectionOnly;
+    bool selectionOnly = false;
 
-    uint firstline;
-    uint lastline;
+    uint firstline = 0;
+    uint lastline = 0;
 
     // Header/Footer Page
-    uint headerHeight;
+    uint headerHeight = 0;
     QStringList headerTagList;
-    uint footerHeight;
+    uint footerHeight = 0;
     QStringList footerTagList;
 
     KTextEditor::Range selectionRange;
@@ -182,7 +170,7 @@ void PrintPainter::updateCache()
 {
     m_fontHeight = m_renderer->fontHeight();
 
-    // figure out the horiizontal space required
+    // figure out the horizontal space required
     QString s = QStringLiteral("%1 ").arg(m_doc->lines());
     s.fill(QLatin1Char('5'), -1); // some non-fixed fonts haven't equally wide numbers
     // FIXME calculate which is actually the widest...
@@ -200,6 +188,7 @@ void PrintPainter::paint(QPrinter *printer) const
     uint y = 0;
     uint currentPage = (printer->fromPage() == 0) ? 1 : printer->fromPage();
     bool pageStarted = true;
+    uint remainder = 0;
 
     // On to draw something :-)
     while (lineCount <= pl.lastline) {
@@ -225,7 +214,6 @@ void PrintPainter::paint(QPrinter *printer) const
             paintLineNumber(painter, lineCount, pl);
         }
 
-        uint remainder = 0;
         paintLine(painter, lineCount, y, remainder, pl);
 
         if (!remainder) {
@@ -264,9 +252,9 @@ void PrintPainter::configure(const QPrinter *printer, PageLayout &pl) const
 
     if (m_useHeader || m_useFooter) {
         // Set up a tag map
-        // This retrieves all tags, ued or not, but
-        // none of theese operations should be expensive,
-        // and searcing each tag in the format strings is avoided.
+        // This retrieves all tags, used or not, but
+        // none of these operations should be expensive,
+        // and searching each tag in the format strings is avoided.
         QDateTime dt = QDateTime::currentDateTime();
         QMap<QString, QString> tags;
 
@@ -337,7 +325,6 @@ void PrintPainter::configure(const QPrinter *printer, PageLayout &pl) const
                 it.setValue(tag);
             }
 
-            // adjust maxheight, so we can know when/where to print footer
             pl.maxHeight -= pl.footerHeight;
         }
     } // if ( useHeader || useFooter )
@@ -362,7 +349,7 @@ void PrintPainter::configure(const QPrinter *printer, PageLayout &pl) const
         pageHeight -= pl.headerHeight + pl.innerMargin;
     }
     if (m_useFooter) {
-        pageHeight -= pl.innerMargin;
+        pageHeight -= pl.footerHeight + pl.innerMargin;
     }
 
     const int linesPerPage = pageHeight / m_fontHeight;
@@ -434,7 +421,7 @@ void PrintPainter::paintNewPage(QPainter &painter, const uint currentPage, uint
 void PrintPainter::paintHeader(QPainter &painter, const uint currentPage, uint &y, const PageLayout &pl) const
 {
     painter.save();
-    painter.setPen(m_headerForeground);
+    painter.setPen(QPen(m_headerForeground, 0.5));
     painter.setFont(m_fhFont);
 
     if (m_useHeaderBackground) {
@@ -474,13 +461,14 @@ void PrintPainter::paintHeader(QPainter &painter, const uint currentPage, uint &
 void PrintPainter::paintFooter(QPainter &painter, const uint currentPage, const PageLayout &pl) const
 {
     painter.save();
-    painter.setPen(m_footerForeground);
+    painter.setPen(QPen(m_footerForeground, 0.5));
+    painter.setFont(m_fhFont);
 
     if (!(m_useFooterBackground || m_useBox || m_useBackground)) {// draw a 1 px (!?) line to separate footer from contents
-        painter.drawLine(0, pl.maxHeight + pl.innerMargin - 1, pl.headerWidth, pl.maxHeight + pl.innerMargin - 1);
+        painter.drawLine(0, pl.pageHeight - pl.footerHeight - 1, pl.headerWidth, pl.pageHeight - pl.footerHeight - 1);
     }
     if (m_useFooterBackground) {
-        painter.fillRect(0, pl.maxHeight + pl.innerMargin + m_boxWidth, pl.headerWidth, pl.footerHeight, m_footerBackground);
+        painter.fillRect(0, pl.pageHeight - pl.footerHeight, pl.headerWidth, pl.footerHeight, m_footerBackground);
     }
 
     if (pl.footerTagList.count() == 3) {
@@ -496,7 +484,7 @@ void PrintPainter::paintFooter(QPainter &painter, const uint currentPage, const
             if (s.indexOf(QLatin1String("%p")) != -1) {
                 s.replace(QLatin1String("%p"), QString::number(currentPage));
             }
-            painter.drawText(marg, pl.maxHeight + pl.innerMargin, pl.headerWidth - (marg * 2), pl.footerHeight, align, s);
+            painter.drawText(marg, pl.pageHeight - pl.footerHeight, pl.headerWidth - (marg * 2), pl.footerHeight, align, s);
             align = Qt::AlignVCenter | (i == 0 ? Qt::AlignHCenter : Qt::AlignRight);
         }
     }
@@ -510,7 +498,7 @@ void PrintPainter::paintGuide(QPainter &painter, uint &y, const PageLayout &pl)
     int _ystart = y;
     QString _hlName = m_doc->highlight()->name();
 
-    QList<KTextEditor::Attribute::Ptr> _attributes; // list of highlight attributes for the legend
+    QVector<KTextEditor::Attribute::Ptr> _attributes; // list of highlight attributes for the legend
     m_doc->highlight()->getKateExtendedAttributeList(m_renderer->config()->schema(), _attributes);
 
     KateAttributeList _defaultAttributes;
@@ -698,6 +686,9 @@ void PrintPainter::paintLine(QPainter &painter, const uint line, uint &y, uint &
     } else if (y + m_fontHeight * _lines > pl.maxHeight) {
         remainder = _lines - ((pl.maxHeight - y) / m_fontHeight);
         painter.setClipRect(0, 0, pl.maxWidth, (_lines - int(remainder)) * m_fontHeight + 1); //### drop the crosspatch in printerfriendly mode???
+    } else if (!pl.selectionOnly) {
+        painter.setClipRegion(QRegion());
+        painter.setClipping(false);
     }
 
     m_renderer->paintTextLine(painter, rangeptr, 0, (int)pl.maxWidth);
